Method: Bunny::Channel#prefetch
- Defined in:
- lib/bunny/channel.rb
#prefetch(count, global = false) ⇒ Object
Sets how many messages will be given to consumers on this channel before they have to acknowledge or reject one of the previously consumed messages
446 447 448 |
# File 'lib/bunny/channel.rb', line 446 def prefetch(count, global = false) self.basic_qos(count, global) end |