Class: Bunny::Queue
- Inherits:
-
Object
- Object
- Bunny::Queue
- Defined in:
- lib/bunny/queue.rb
Overview
Represents AMQP 0.9.1 queue.
Defined Under Namespace
Instance Attribute Summary collapse
-
#channel ⇒ Bunny::Channel
readonly
Channel this queue uses.
-
#name ⇒ String
readonly
Queue name.
-
#options ⇒ Hash
readonly
Options this queue was created with.
Class Method Summary collapse
Instance Method Summary collapse
-
#arguments ⇒ Hash
Additional optional arguments (typically used by RabbitMQ extensions and plugins).
-
#auto_delete? ⇒ Boolean
True if this queue was declared as automatically deleted (deleted as soon as last consumer unbinds).
-
#bind(exchange, opts = {}) ⇒ Object
Binds queue to an exchange.
-
#consumer_count ⇒ Integer
How many active consumers the queue has.
- #declare! ⇒ Object
-
#delete(opts = {}) ⇒ Object
Deletes the queue.
-
#durable? ⇒ Boolean
True if this queue was declared as durable (will survive broker restart).
-
#exclusive? ⇒ Boolean
True if this queue was declared as exclusive (limited to just one consumer).
-
#initialize(channel, name = AMQ::Protocol::EMPTY_STRING, opts = {}) ⇒ Queue
constructor
A new instance of Queue.
- #inspect ⇒ Object
-
#message_count ⇒ Integer
How many messages the queue has ready (e.g. not delivered but not unacknowledged).
-
#pop(opts = { manual_ack: false }, &block) ⇒ Array
(also: #get)
Triple of delivery info, message properties and message content.
-
#publish(payload, opts = {}) ⇒ Object
Publishes a message to the queue via default exchange.
-
#purge(opts = {}) ⇒ Object
Purges a queue (removes all messages from it).
-
#server_named? ⇒ Boolean
True if this queue was declared as server named.
-
#status ⇒ Hash
A hash with information about the number of queue messages and consumers.
-
#subscribe(opts = { consumer_tag: @channel.generate_consumer_tag, manual_ack: false, exclusive: false, block: false, on_cancellation: nil }, &block) ⇒ Object
Adds a consumer to the queue (subscribes for message deliveries).
-
#subscribe_with(consumer, opts = { block: false }) ⇒ Object
Adds a consumer object to the queue (subscribes for message deliveries).
- #to_s ⇒ Object
-
#unbind(exchange, opts = {}) ⇒ Object
Unbinds queue from an exchange.
- #update_name_to(value) ⇒ Object
Constructor Details
#initialize(channel, name = AMQ::Protocol::EMPTY_STRING, opts = {}) ⇒ Queue
Returns a new instance of Queue.
63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 |
# File 'lib/bunny/queue.rb', line 63 def initialize(channel, name = AMQ::Protocol::EMPTY_STRING, opts = {}) # old Bunny versions pass a connection here. In that case, # we just use default channel from it. MK. @channel = channel @name = name @options = self.class.(name, opts) @durable = @options[:durable] @exclusive = @options[:exclusive] @server_named = @name.empty? @auto_delete = @options[:auto_delete] @type = @options[:type] @arguments = if @type and !@type.empty? then (@options[:arguments] || {}).merge({XArgs::QUEUE_TYPE => @type}) else @options[:arguments] end verify_type!(@arguments) # reassigns updated and verified arguments because Bunny::Channel#declare_queue # accepts a map of options @options[:arguments] = @arguments @bindings = Array.new @default_consumer = nil declare! unless opts[:no_declare] # for basic.deliver dispatch and such @channel.register_queue(self) # for topology recovery @channel.record_queue(self) end |
Instance Attribute Details
#channel ⇒ Bunny::Channel (readonly)
Returns Channel this queue uses.
43 44 45 |
# File 'lib/bunny/queue.rb', line 43 def channel @channel end |
#name ⇒ String (readonly)
Returns Queue name.
45 46 47 |
# File 'lib/bunny/queue.rb', line 45 def name @name end |
#options ⇒ Hash (readonly)
Returns Options this queue was created with.
47 48 49 |
# File 'lib/bunny/queue.rb', line 47 def @options end |
Class Method Details
.add_default_options(name, opts) ⇒ Object
398 399 400 401 402 403 404 405 406 407 408 409 410 411 412 413 |
# File 'lib/bunny/queue.rb', line 398 def self.(name, opts) # :nowait is always false for Bunny h = { queue: name, nowait: false }.merge(opts) if name.empty? { passive: false, durable: false, exclusive: false, auto_delete: false, arguments: nil }.merge(h) else h end end |
.verify_type!(args0 = {}) ⇒ Object
377 378 379 380 381 382 383 |
# File 'lib/bunny/queue.rb', line 377 def self.verify_type!(args0 = {}) # be extra defensive args = args0 || {} q_type = args["x-queue-type"] || args[:"x-queue-type"] raise ArgumentError, "unsupported queue type #{q_type.inspect}, supported ones: #{Types::KNOWN.join(', ')}" if (q_type and !Types.known?(q_type)) end |
Instance Method Details
#arguments ⇒ Hash
Returns Additional optional arguments (typically used by RabbitMQ extensions and plugins).
128 129 130 |
# File 'lib/bunny/queue.rb', line 128 def arguments @arguments end |
#auto_delete? ⇒ Boolean
Returns true if this queue was declared as automatically deleted (deleted as soon as last consumer unbinds).
115 116 117 |
# File 'lib/bunny/queue.rb', line 115 def auto_delete? @auto_delete end |
#bind(exchange, opts = {}) ⇒ Object
Binds queue to an exchange
159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 |
# File 'lib/bunny/queue.rb', line 159 def bind(exchange, opts = {}) @channel.queue_bind(@name, exchange, opts) exchange_name = if exchange.is_a?(Bunny::Exchange) exchange.name else exchange end # store bindings for automatic recovery. We need to be very careful to # not cause an infinite rebinding loop here when we recover. MK. binding = { exchange: exchange_name, routing_key: (opts[:routing_key] || opts[:key]), arguments: opts[:arguments] } @bindings.push(binding) unless @bindings.include?(binding) self end |
#consumer_count ⇒ Integer
Returns How many active consumers the queue has.
372 373 374 375 |
# File 'lib/bunny/queue.rb', line 372 def consumer_count s = self.status s[:consumer_count] end |
#declare! ⇒ Object
390 391 392 393 |
# File 'lib/bunny/queue.rb', line 390 def declare! queue_declare_ok = @channel.queue_declare(@name, @options) @name = queue_declare_ok.queue end |
#delete(opts = {}) ⇒ Object
Deletes the queue
341 342 343 344 345 |
# File 'lib/bunny/queue.rb', line 341 def delete(opts = {}) @channel.delete_recorded_queue_named(self.name) @channel.deregister_queue(self) @channel.queue_delete(@name, opts) end |
#durable? ⇒ Boolean
Returns true if this queue was declared as durable (will survive broker restart).
101 102 103 |
# File 'lib/bunny/queue.rb', line 101 def durable? @durable end |
#exclusive? ⇒ Boolean
Returns true if this queue was declared as exclusive (limited to just one consumer).
108 109 110 |
# File 'lib/bunny/queue.rb', line 108 def exclusive? @exclusive end |
#inspect ⇒ Object
144 145 146 |
# File 'lib/bunny/queue.rb', line 144 def inspect to_s end |
#message_count ⇒ Integer
Returns How many messages the queue has ready (e.g. not delivered but not unacknowledged).
366 367 368 369 |
# File 'lib/bunny/queue.rb', line 366 def s = self.status s[:message_count] end |
#pop(opts = { manual_ack: false }, &block) ⇒ Array Also known as: get
Returns Triple of delivery info, message properties and message content. If the queue is empty, all three will be nils.
290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 306 307 308 309 310 311 312 313 314 315 316 |
# File 'lib/bunny/queue.rb', line 290 def pop(opts = { manual_ack: false }, &block) unless opts[:ack].nil? warn "[DEPRECATION] `:ack` is deprecated. Please use `:manual_ack` instead." opts[:manual_ack] = opts[:ack] end get_response, properties, content = @channel.basic_get(@name, opts) if block if properties di = GetResponse.new(get_response, @channel) mp = MessageProperties.new(properties) block.call(di, mp, content) else block.call(nil, nil, nil) end else if properties di = GetResponse.new(get_response, @channel) mp = MessageProperties.new(properties) [di, mp, content] else [nil, nil, nil] end end end |
#publish(payload, opts = {}) ⇒ Object
Publishes a message to the queue via default exchange. Takes the same arguments as Exchange#publish
325 326 327 328 329 |
# File 'lib/bunny/queue.rb', line 325 def publish(payload, opts = {}) @channel.default_exchange.publish(payload, opts.merge(routing_key: @name)) self end |
#purge(opts = {}) ⇒ Object
Purges a queue (removes all messages from it)
350 351 352 353 354 |
# File 'lib/bunny/queue.rb', line 350 def purge(opts = {}) @channel.queue_purge(@name, opts) self end |
#server_named? ⇒ Boolean
Returns true if this queue was declared as server named.
122 123 124 |
# File 'lib/bunny/queue.rb', line 122 def server_named? @server_named end |
#status ⇒ Hash
Returns A hash with information about the number of queue messages and consumers.
359 360 361 362 363 |
# File 'lib/bunny/queue.rb', line 359 def status queue_declare_ok = @channel.queue_declare(@name, @options.merge(passive: true)) { message_count: queue_declare_ok., consumer_count: queue_declare_ok.consumer_count } end |
#subscribe(opts = { consumer_tag: @channel.generate_consumer_tag, manual_ack: false, exclusive: false, block: false, on_cancellation: nil }, &block) ⇒ Object
Adds a consumer to the queue (subscribes for message deliveries).
216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 |
# File 'lib/bunny/queue.rb', line 216 def subscribe(opts = { consumer_tag: @channel.generate_consumer_tag, manual_ack: false, exclusive: false, block: false, on_cancellation: nil }, &block) unless opts[:ack].nil? warn "[DEPRECATION] `:ack` is deprecated. Please use `:manual_ack` instead." opts[:manual_ack] = opts[:ack] end ctag = opts.fetch(:consumer_tag, @channel.generate_consumer_tag) consumer = Consumer.new(@channel, self, ctag, !opts[:manual_ack], opts[:exclusive], opts[:arguments]) consumer.on_delivery(&block) consumer.on_cancellation(&opts[:on_cancellation]) if opts[:on_cancellation] @channel.basic_consume_with(consumer) if opts[:block] # joins current thread with the consumers pool, will block # the current thread for as long as the consumer pool is active @channel.work_pool.join end consumer end |
#subscribe_with(consumer, opts = { block: false }) ⇒ Object
Adds a consumer object to the queue (subscribes for message deliveries).
259 260 261 262 263 264 |
# File 'lib/bunny/queue.rb', line 259 def subscribe_with(consumer, opts = { block: false }) @channel.basic_consume_with(consumer) @channel.work_pool.join if opts[:block] consumer end |
#to_s ⇒ Object
139 140 141 142 |
# File 'lib/bunny/queue.rb', line 139 def to_s oid = ("0x%x" % (self.object_id << 1)) "<#{self.class.name}:#{oid} @name=\"#{name}\" channel=#{@channel.to_s} @durable=#{@durable} @auto_delete=#{@auto_delete} @exclusive=#{@exclusive} @arguments=#{@arguments}>" end |
#unbind(exchange, opts = {}) ⇒ Object
Unbinds queue from an exchange
188 189 190 191 192 193 194 195 196 197 198 199 200 201 |
# File 'lib/bunny/queue.rb', line 188 def unbind(exchange, opts = {}) @channel.queue_unbind(@name, exchange, opts) exchange_name = if exchange.is_a?(Bunny::Exchange) exchange.name else exchange end @bindings.delete_if { |b| b[:exchange] == exchange_name && b[:routing_key] == (opts[:routing_key] || opts[:key]) && b[:arguments] == opts[:arguments] } self end |
#update_name_to(value) ⇒ Object
134 135 136 137 |
# File 'lib/bunny/queue.rb', line 134 def update_name_to(value) @name = value self end |