Class: Bunny::Queue
- Inherits:
-
Object
- Object
- Bunny::Queue
- Defined in:
- lib/bunny/queue.rb
Overview
Represents AMQP 0.9.1 queue.
Instance Attribute Summary collapse
-
#channel ⇒ Bunny::Channel
readonly
Channel this queue uses.
-
#name ⇒ String
readonly
Queue name.
-
#options ⇒ Hash
readonly
Options this queue was created with.
Instance Method Summary collapse
-
#arguments ⇒ Hash
Additional optional arguments (typically used by RabbitMQ extensions and plugins).
-
#auto_delete? ⇒ Boolean
True if this queue was declared as automatically deleted (deleted as soon as last consumer unbinds).
-
#bind(exchange, opts = {}) ⇒ Object
Binds queue to an exchange.
-
#consumer_count ⇒ Integer
How many active consumers the queue has.
-
#delete(opts = {}) ⇒ Object
Deletes the queue.
-
#durable? ⇒ Boolean
True if this queue was declared as durable (will survive broker restart).
-
#exclusive? ⇒ Boolean
True if this queue was declared as exclusive (limited to just one consumer).
-
#initialize(channel, name = AMQ::Protocol::EMPTY_STRING, opts = {}) ⇒ Queue
constructor
A new instance of Queue.
- #inspect ⇒ Object
-
#message_count ⇒ Integer
How many messages the queue has ready (e.g. not delivered but not unacknowledged).
-
#pop(opts = {:manual_ack => false}, &block) ⇒ Array
(also: #get)
Triple of delivery info, message properties and message content.
-
#publish(payload, opts = {}) ⇒ Object
Publishes a message to the queue via default exchange.
-
#purge(opts = {}) ⇒ Object
Purges a queue (removes all messages from it).
-
#server_named? ⇒ Boolean
True if this queue was declared as server named.
-
#status ⇒ Hash
A hash with information about the number of queue messages and consumers.
-
#subscribe(opts = { :consumer_tag => @channel.generate_consumer_tag, :manual_ack => false, :exclusive => false, :block => false, :on_cancellation => nil }, &block) ⇒ Object
Adds a consumer to the queue (subscribes for message deliveries).
-
#subscribe_with(consumer, opts = {:block => false}) ⇒ Object
Adds a consumer object to the queue (subscribes for message deliveries).
- #to_s ⇒ Object
-
#unbind(exchange, opts = {}) ⇒ Object
Unbinds queue from an exchange.
Constructor Details
#initialize(channel, name = AMQ::Protocol::EMPTY_STRING, opts = {}) ⇒ Queue
Returns a new instance of Queue.
34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 |
# File 'lib/bunny/queue.rb', line 34 def initialize(channel, name = AMQ::Protocol::EMPTY_STRING, opts = {}) # old Bunny versions pass a connection here. In that case, # we just use default channel from it. MK. @channel = channel @name = name = self.class.(name, opts) @consumers = Hash.new @durable = [:durable] @exclusive = [:exclusive] @server_named = @name.empty? @auto_delete = [:auto_delete] @arguments = [:arguments] @bindings = Array.new @default_consumer = nil declare! unless opts[:no_declare] @channel.register_queue(self) end |
Instance Attribute Details
#channel ⇒ Bunny::Channel (readonly)
Returns Channel this queue uses.
15 16 17 |
# File 'lib/bunny/queue.rb', line 15 def channel @channel end |
#name ⇒ String (readonly)
Returns Queue name.
17 18 19 |
# File 'lib/bunny/queue.rb', line 17 def name @name end |
#options ⇒ Hash (readonly)
Returns Options this queue was created with.
19 20 21 |
# File 'lib/bunny/queue.rb', line 19 def end |
Instance Method Details
#arguments ⇒ Hash
Returns Additional optional arguments (typically used by RabbitMQ extensions and plugins).
87 88 89 |
# File 'lib/bunny/queue.rb', line 87 def arguments @arguments end |
#auto_delete? ⇒ Boolean
Returns true if this queue was declared as automatically deleted (deleted as soon as last consumer unbinds).
74 75 76 |
# File 'lib/bunny/queue.rb', line 74 def auto_delete? @auto_delete end |
#bind(exchange, opts = {}) ⇒ Object
Binds queue to an exchange
111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 |
# File 'lib/bunny/queue.rb', line 111 def bind(exchange, opts = {}) @channel.queue_bind(@name, exchange, opts) exchange_name = if exchange.respond_to?(:name) exchange.name else exchange end # store bindings for automatic recovery. We need to be very careful to # not cause an infinite rebinding loop here when we recover. MK. binding = { :exchange => exchange_name, :routing_key => (opts[:routing_key] || opts[:key]), :arguments => opts[:arguments] } @bindings.push(binding) unless @bindings.include?(binding) self end |
#consumer_count ⇒ Integer
Returns How many active consumers the queue has.
324 325 326 327 |
# File 'lib/bunny/queue.rb', line 324 def consumer_count s = self.status s[:consumer_count] end |
#delete(opts = {}) ⇒ Object
Deletes the queue
294 295 296 297 |
# File 'lib/bunny/queue.rb', line 294 def delete(opts = {}) @channel.deregister_queue(self) @channel.queue_delete(@name, opts) end |
#durable? ⇒ Boolean
Returns true if this queue was declared as durable (will survive broker restart).
60 61 62 |
# File 'lib/bunny/queue.rb', line 60 def durable? @durable end |
#exclusive? ⇒ Boolean
Returns true if this queue was declared as exclusive (limited to just one consumer).
67 68 69 |
# File 'lib/bunny/queue.rb', line 67 def exclusive? @exclusive end |
#inspect ⇒ Object
96 97 98 |
# File 'lib/bunny/queue.rb', line 96 def inspect to_s end |
#message_count ⇒ Integer
Returns How many messages the queue has ready (e.g. not delivered but not unacknowledged).
318 319 320 321 |
# File 'lib/bunny/queue.rb', line 318 def s = self.status s[:message_count] end |
#pop(opts = {:manual_ack => false}, &block) ⇒ Array Also known as: get
Returns Triple of delivery info, message properties and message content. If the queue is empty, all three will be nils.
243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 |
# File 'lib/bunny/queue.rb', line 243 def pop(opts = {:manual_ack => false}, &block) unless opts[:ack].nil? warn "[DEPRECATION] `:ack` is deprecated. Please use `:manual_ack` instead." opts[:manual_ack] = opts[:ack] end get_response, properties, content = @channel.basic_get(@name, opts) if block if properties di = GetResponse.new(get_response, properties, @channel) mp = MessageProperties.new(properties) block.call(di, mp, content) else block.call(nil, nil, nil) end else if properties di = GetResponse.new(get_response, properties, @channel) mp = MessageProperties.new(properties) [di, mp, content] else [nil, nil, nil] end end end |
#publish(payload, opts = {}) ⇒ Object
Publishes a message to the queue via default exchange. Takes the same arguments as Exchange#publish
278 279 280 281 282 |
# File 'lib/bunny/queue.rb', line 278 def publish(payload, opts = {}) @channel.default_exchange.publish(payload, opts.merge(:routing_key => @name)) self end |
#purge(opts = {}) ⇒ Object
Purges a queue (removes all messages from it)
302 303 304 305 306 |
# File 'lib/bunny/queue.rb', line 302 def purge(opts = {}) @channel.queue_purge(@name, opts) self end |
#server_named? ⇒ Boolean
Returns true if this queue was declared as server named.
81 82 83 |
# File 'lib/bunny/queue.rb', line 81 def server_named? @server_named end |
#status ⇒ Hash
Returns A hash with information about the number of queue messages and consumers.
311 312 313 314 315 |
# File 'lib/bunny/queue.rb', line 311 def status queue_declare_ok = @channel.queue_declare(@name, .merge(:passive => true)) {:message_count => queue_declare_ok., :consumer_count => queue_declare_ok.consumer_count} end |
#subscribe(opts = { :consumer_tag => @channel.generate_consumer_tag, :manual_ack => false, :exclusive => false, :block => false, :on_cancellation => nil }, &block) ⇒ Object
Adds a consumer to the queue (subscribes for message deliveries).
169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 |
# File 'lib/bunny/queue.rb', line 169 def subscribe(opts = { :consumer_tag => @channel.generate_consumer_tag, :manual_ack => false, :exclusive => false, :block => false, :on_cancellation => nil }, &block) unless opts[:ack].nil? warn "[DEPRECATION] `:ack` is deprecated. Please use `:manual_ack` instead." opts[:manual_ack] = opts[:ack] end ctag = opts.fetch(:consumer_tag, @channel.generate_consumer_tag) consumer = Consumer.new(@channel, self, ctag, !opts[:manual_ack], opts[:exclusive], opts[:arguments]) consumer.on_delivery(&block) consumer.on_cancellation(&opts[:on_cancellation]) if opts[:on_cancellation] @channel.basic_consume_with(consumer) if opts[:block] # joins current thread with the consumers pool, will block # the current thread for as long as the consumer pool is active @channel.work_pool.join end consumer end |
#subscribe_with(consumer, opts = {:block => false}) ⇒ Object
Adds a consumer object to the queue (subscribes for message deliveries).
212 213 214 215 216 217 |
# File 'lib/bunny/queue.rb', line 212 def subscribe_with(consumer, opts = {:block => false}) @channel.basic_consume_with(consumer) @channel.work_pool.join if opts[:block] consumer end |
#to_s ⇒ Object
91 92 93 94 |
# File 'lib/bunny/queue.rb', line 91 def to_s oid = ("0x%x" % (self.object_id << 1)) "<#{self.class.name}:#{oid} @name=\"#{name}\" channel=#{@channel.to_s} @durable=#{@durable} @auto_delete=#{@auto_delete} @exclusive=#{@exclusive} @arguments=#{@arguments}>" end |
#unbind(exchange, opts = {}) ⇒ Object
Unbinds queue from an exchange
140 141 142 143 144 145 146 147 148 149 150 151 152 153 |
# File 'lib/bunny/queue.rb', line 140 def unbind(exchange, opts = {}) @channel.queue_unbind(@name, exchange, opts) exchange_name = if exchange.respond_to?(:name) exchange.name else exchange end @bindings.delete_if { |b| b[:exchange] == exchange_name && b[:routing_key] == (opts[:routing_key] || opts[:key]) && b[:arguments] == opts[:arguments] } self end |